<?php
if (!defined('PLX_ROOT')) { exit; };

/*
 * Plugin pour HighlightJS
 * https://github.com/highlightjs/highlight.js
 *
 * Permet la coloration syntaxique d'un extrait de code.
 * L'extrait doit être placé entre la paire de couples de balises :
 *
 * <pre><code lang="hljs php"> et </code></pre>
 *
 * Replace php with an other language if needed.
 * Available languages in common usage :
 * */

class kzHighlightJS extends plxPlugin {

	const BASE = PLX_PLUGINS . __CLASS__ . '/';
	const LIB = self::BASE . 'highlight.js/build/';
	const STYLES_LIST = PLX_PLUGINS . __CLASS__ . '/styles.txt';
	const PARAM1 = 'theme';

	const SITE_HOOKS = array(
		'ThemeEndHead',
        'ThemeEndBody',
        'plxMotorParseArticle',
        'plxShowStaticContentBegin',
	);

	const BEGIN_CODE = '<?php' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';

	/* For replacing each '<' by entity inside <pre><code> tags in the content of an article or a static page */
	const PATTERN = '@(<pre[^>]*>\s*<code[^>]*>)\s*(.*?)\s*(</code>\s*</pre>)@is';

	public $themes = false;

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		parent::setConfigProfil(PROFIL_ADMIN);

		$display_help = (basename(parse_url($_SERVER['REQUEST_URI'],  PHP_URL_PATH)) == 'parametres_help.php');
		if(!defined('PLX_ADMIN')) {
			foreach(self::SITE_HOOKS as $hook) {
				parent::addHook($hook, $hook);
			}
		} elseif(
			basename($_SERVER['REQUEST_URI']) == 'parametres_plugin.php?p=' . __CLASS__ or
			$display_help
		) {
			// Just use for parametres_plugin.php in admin mode
			$this->themes = array();

			if(file_exists(self::STYLES_LIST)) {
				foreach(file(self::STYLES_LIST, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) as $k) {
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
			}

			if(empty($this->themes)) {
				foreach(glob(self::LIB . 'demo/styles/*.css') as $t) {
					$k = basename($t, '.css');
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
				if(!empty($this->themes) and is_writable(__DIR__)) {
					file_put_contents(self::STYLES_LIST, implode(PHP_EOL, array_keys($this->themes)));
				}
			}

			parent::addHook('AdminTopEndHead', $display_help ? 'ThemeEndHead' : 'AdminTopEndHead');
			parent::addHook('AdminFootEndBody', 'ThemeEndBody');
		}
	}

	public function OnActivate() {
		// Lists again all available themes
		if(file_exists(self::STYLES_LIST) and is_writable(__DIR__)) {
			unlink(self::STYLES_LIST);
		}
	}

	/*
	 * Fonction de callback pour preg_replace()
	 * */
	static public function specialChars($matches) {
		return $matches[1].strtr($matches[2], array('<' => '&lt;', '?>' => '?&gt;')) . $matches[3];
	}

	/*
	 * Converts '<' and '>' into HTML entities.
	 * */
	public function plxMotorParseArticle() {
		echo self::BEGIN_CODE;
?>
$art['content'] = preg_replace_callback(
	'<?= self::PATTERN ?>',
	array('<?= __CLASS__ ?>', 'specialChars'),
	$art['content']
);
<?php
		echo self::END_CODE;
	}

	/*
	 * Converts '<' and '>' into HTML entities.
	 * */
	public function plxShowStaticContentBegin() {
		echo self::BEGIN_CODE;
?>
if($this->plxMotor->aStats[ $this->plxMotor->cible ]['readable'] != 1) {
	echo '<p><?= L_STATICCONTENT_INPROCESS ?></p>';
	return true;
} else {
	$filename =
		PLX_ROOT .
		$this->plxMotor->aConf['racine_statiques'] .
		$this->plxMotor->cible . '.' .
		$this->plxMotor->aStats[ $this->plxMotor->cible ]['url'] .
		'.php';
	$content = file_get_contents($filename);

	$pattern = '<?= self::PATTERN ?>';
	if(preg_match($pattern, $content)) {
		$new_content = preg_replace_callback($pattern, array('<?= __CLASS__ ?>', 'specialChars'), $content);
		ob_start();
		eval('<?= self::END_CODE ?>' . $new_content . '<?= self::BEGIN_CODE ?>');
		$output = ob_get_clean();
		eval($this->plxMotor->plxPlugins->callHook('plxShowStaticContent'));
		echo $output;
		return true;
	}
	return false;
}
<?php
		echo self::END_CODE;
	}

	public function AdminTopEndHead() {
		// using alternate attribute avoids for loading all stylesheets if not required
		$currentStyle = $this->getParam(self::PARAM1);
		if(empty($currentStyle)) { $currentStyle = 'default'; }
		foreach($this->themes as $k=>$v) {
			$href = self::LIB."demo/styles/$k.css";
?>
	<link rel="<?= ($k != $currentStyle) ? 'alternate ' : '' ?>stylesheet" type="text/css" href="<?= $href ?>" title="<?= $k ?>" disabled />
<?php
		}
	}

	public function ThemeEndHead() {
		$param = trim($this->getParam(self::PARAM1));
		if(empty($param)) { $param = 'default'; }
		$href = 'demo/styles/' . $param . '.css';
		if(file_exists(self::LIB . $href)) {
?>
	<link rel="stylesheet" type="text/css" href="<?= self::LIB . $href ?>" />
<?php
		}
	}

	public function ThemeEndBody() {
		// https://highlightjs.org/usage/
		// Also in use for config.php for selecting a theme

		$t = glob(self::BASE . 'jquery*.js');
		if(empty($t)) {
?>
<!-- jQuery not found in <?= self::BASE ?> folder -->
<?php
			return;
		}

		// "pack" for version < 10.0, otherwise "min" for highlight library
		foreach(array('min', 'pack') as $k) {
			$filename = 'highlight.' . $k . '.js';
			if(file_exists(self::LIB . $filename)) {
?>
	<script type="text/javascript" src="<?= self::LIB . $filename ?>"></script>
<?php
				break;
			}
		}
?>
	<script type="text/javascript" src="<?= self::BASE . __CLASS__ ?>.js" data-src="<?= $t[0] ?>"></script>
<?php
	}

}
?>
